/*
 *    Copyright  1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */

#include "config.h"
#include <X11/Xos.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>

#include <stdio.h>
#include <time.h>
#include <errno.h>
#include "xmap.h"
#include "gps.h"
#include "mode.h"
#include "debug.h"

static FILE *logfp = NULL;
static FILE *ilogfp = NULL;
static char logbuf[128];
static char oldbuf[128];
static int visible = 0;

extern Widget status_2;

static void make_visible()
{
	Dimension width, height;
	Position x, y;

	XtVaGetValues( status_2,
		XmNwidth, &width,
		XmNheight, &height,
		NULL);

        /* Set to dest x and y clip origin */
        XSetClipOrigin(XtDisplay(status_2), status2GC, 
		width - 15, (height - 9)/2);

        /* Set the clipping mask */
        XSetClipMask(XtDisplay(status_2), status2GC, get_mark_mask(6));

	XCopyArea(XtDisplay(status_2), get_mark_pixmap(6),
		XtWindow(status_2), status2GC,
		0, 0,
		9, 9,
		width - 15, (height - 9)/2);

        XSetClipMask(XtDisplay(status_2), status2GC, 0);
}

static void make_invisible()
{
	Dimension width, height;
	Position x, y;

	XtVaGetValues( status_2,
		XmNwidth, &width,
		XmNheight, &height,
		NULL);

	XClearArea(XtDisplay(status_2), XtWindow(status_2),
		width - 15, (height - 9)/2, 9, 9, False);
}

void show_log_status(void)
{
	if (!visible) make_visible();
	else make_invisible();
	visible ^= 1;
}

//
// Open a log file and write some header information.
//

open_log()
{
    if (logfp) {
	// Already open!
	beep();
	return;
    }
    select_log_file();
}

new_log(char *fn)
{
    time_t thistime;

    // if the file can be opend, do it
    logfp = fopen(fn, "a");

    // else tell the user and cancel
    if (!logfp) {
	char bufp[512];
	snprintf(bufp, 511, "Can't open \"%s\"", fn);
	error(bufp, errno);
	return;
    }
    // A side effect of loging is that we need tracking as well.
    // If not already on, turn it on now.

    ch_mode(LOGTODISK, START);
    if (gpsdata.status == -1) {
	track_start();
    }
    time(&thistime);
    fprintf(logfp, "hugolog 1.0 %s", asctime(gmtime(&thistime)));

    make_visible();
}

read_log(char *fn)
{
    time_t thistime;

    // if the file can be opend, do it
    ilogfp = fopen(fn, "r");

    // else tell the user and cancel
    if (!ilogfp) {
	char bufp[512];
	snprintf(bufp, 511, "Can't open \"%s\"", fn);
	error(bufp, errno);
	return;
    }
    make_visible();
    replay(fn, ilogfp);
    fclose(ilogfp);
    ilogfp = NULL;
    make_invisible();
}

//
// Is called once a second. Write pertinent data to log, if loging is on.
//

do_log()
{
    if (logfp && gpsdata.status > 0) {
	// We just prepare the ascii log entry, but write only
	// if it is not the same as last time.

	sprintf(logbuf, "%d %f %f %.1f %.0f",
		gpsdata.status,
		gpsdata.latitude, gpsdata.longitude,
		gpsdata.velocity, gpsdata.altitude);

	if (strcmp(logbuf, oldbuf)) {
	    fprintf(logfp, "%s %s %s\n", logbuf,
		    gpsdata.date, gpsdata.time);
	    strcpy(oldbuf, logbuf);
	}
	if (visible) {
		make_invisible();
		visible = 0;
	}
	else {
		make_visible();
		visible = 1;
	}
    }
}


//
// Close the log file after flushing any buffered entries.
//

close_log()
{
    time_t thistime;

    if (logfp) {
	if (oldbuf[0]) {
	    fprintf(logfp, "%s %s %s\n", logbuf,
		    gpsdata.date, gpsdata.time);
	}
	time(&thistime);
	fprintf(logfp, "end %s", asctime(gmtime(&thistime)));
	fclose(logfp);
	logfp = NULL;
	oldbuf[0] = '\0';
	make_invisible();
	ch_mode(LOGTODISK, STOP);
    }
}


track_from_log()
{
    select_ilog_file();
}
